﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

namespace GPD
{
    public struct setVehicle_
    {

        public string manufacturer;
        public string model;
        public int year;
        public float consumption;
        public string fueltype;
    };

    public static class DatabaseHandler
    {
        private static DBhandlerDataContext db_ = new DBhandlerDataContext();

       
        /**
         *bool addUser adds a new user when he/she registers
         *
         * throws: LogicException if email or password are empty
         * 
         * preconditions: Email and password must not be empty
         * postcondition: User is added to database
         * 
         * parameters: string email address is users
         *              email address. Presumed to be
         *              valid.
         *              
         *              string password is users password
         *              is presumed to be valid
         *              
         *             
         */

        public static bool addUser(string email, string password)
        {

            if( email == "" || password == "")
            {
                throw( new LogicException() );
            }

            try
            {
                PERSONID person = new PERSONID();
                person.email = email;
                person.password = password;
                person.firstname = "";
                person.gpd_index = 0;
                person.lastname = "";
                person.nationality = "";
                person.sex = "";
                person.vehicle_consumption = 0;
                person.vehicle_fueltype = "";
                person.vehicle_manufacturer = "";
                person.vehicle_model = "";
                person.vehicle_year = 0;

                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

                if (found.Count() > 0)
                {
                    return false;
                }

                db_.PERSONIDs.InsertOnSubmit(person);
                db_.SubmitChanges();
            }
            catch (Exception e) {
                System.Console.Write(e.StackTrace);
            }
            
            return true;

        }

        /**
         * 
         * editFirstName changes users first name
         * 
         * Throws: LogicException if >1 of the parameters are empty
         * Preconditions: parameters must be assigned a value
         * Postconditions: New first name is set to database
         * 
         * Parameters:   string email is users email address
         *                database uses this as a primary key
         *               string name is users first name
         */

        public static void editFirstName(string email, string name)
        {
            if (email == "" || name == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();
                precessor.firstname = name;
                db_.SubmitChanges();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }
        
        /**
        * 
        * editLastName changes users last name
        * 
        * Throws: LogicException if >1 of the parameters are empty
        * Preconditions: parameters must be assigned a value
        * Postconditions: New last name is set to database
        * 
        * Parameters:   string email is users email address
         *               database uses this as a primary key
        *               string name is users last name
        */
        public static void editLastName(string email, string name)
        {
            if (email == "" || name == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();
                precessor.lastname = name;
                db_.SubmitChanges();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }
        
        /**
       * 
       * editNationality changes users nationality
       * 
       * Throws: LogicException if one ore more parameters are empty
       * Preconditions: parameter email must be assigned a value
       *               parameter nationality must be assigned a value
       * 
       * Postconditions: New nationality is set to database
       * 
       * Parameters:   string email is users email address
       *                database uses this as a primary key
       *               nationality  is the nationality of user
       */
        public static void editNationality(string email, string nationality)
        {
            if (email == "" || nationality == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();
                precessor.nationality = nationality;
                db_.SubmitChanges();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }
        
        /**
      * 
      * editSex changes users Sex
      * 
      * Throws: LogicException if one ore more parameters are empty
      * Preconditions: parameter email must be assigned a value
      *               parameter sex must be assigned a value
      * 
      * Postconditions: New sex is set to database
      * 
      * Parameters:   string email is users email address
      *                database uses this as a primary key
      *               sex date is the sex of a user
      */
        public static void editSex(string email, string sex)
        {
            if (email == "" || sex == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();
                precessor.sex = sex;
                db_.SubmitChanges();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }

        /**
         * 
         * editVechileinfo changes users Vechileinfo
         * 
         * Throws: LogicException email is empty
         * Preconditions: email must not be empty
        * Postconditions: New info is set to database
        * 
        * Parameters:   string email is users email address
        *                database uses this as a primary key
        *               string manufacturer is a manufacturer of a car
         *               string model is a car model
         *               int year is a car models year
         *               float consumption is cars fuelconsumption
         *               string fueltype is a type of fuel car uses
        */
        public static void editVehicleInfo(string email, string manufacturer, string model, int year, float consumption, string fueltype)
        {
            if (email == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();

                if (manufacturer != "")
                {
                    precessor.vehicle_manufacturer = manufacturer;
                }
                if (model != "")
                {
                    precessor.vehicle_model = model;
                }
             
                precessor.vehicle_year = year;
               
                if (fueltype != "")
                {
                    precessor.vehicle_fueltype = fueltype;
                }
                if (consumption != 0.0)
                {
                    precessor.vehicle_consumption = consumption;
                }

                db_.SubmitChanges();
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }
        }
        /**
         * 
         * login logs the user in
         * 
         * Throws: LogicException if one ore more parameters are empty
         * Preconditions: email must not be empty 
         *                  password must not be empty
         *                  
         * 
        * 
        * Parameters:   string email is users email address
        *                database uses this as a primary key
        *                string password is the attempted password
         *               It is validated from database
         *               
         * Returns:     true if login is valid
         *              false if email and password don't match
        */
        public static bool login(string email, string password)
        {
            if (email == "" || password == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

                foreach (var result in found)
                {
                    if (result.password.ToString() == password)
                    {
                        return true;
                    }
                }
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }

            return false;
        }
        
        /**
         * 
         * addPoints adds the given points
         * 
         * Throws: LogicException if email is empty
         * Preconditions: email must not be empty 
         * Postcondition: points are added to database               
        * 
        * Parameters:   string email is users email address
        *                database uses this as a primary key
        *                int points is the points added to database
         *               
        */
        public static void addPoints(int points, string email)
        {
            if (email == "")
            {
                throw (new LogicException());
            }

            try
            {
                var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);
                PERSONID precessor = found.First();
                precessor.gpd_index += points;
                db_.SubmitChanges();
                
            }
            catch (Exception e)
            {
                Console.Write(e.StackTrace);
            }

        }

        // Palauttaa kymmenen parasta hihhulia
        // parissa string koko nimi ja int pisteet tai jotain
        public static List<Pair> getTop()
        {
            var tulo = db_.ExecuteQuery<PERSONID>
                (@"SELECT TOP 10 * FROM PERSONID ORDER BY gpd_index ASC");
            
            List<Pair> top = new List<Pair>();

            foreach (PERSONID id in tulo)
            {
                Pair pair = new Pair();
                pair.First = ( id.firstname + " " + id.lastname);
                pair.Second = (id.gpd_index );
                top.Add(pair);
            }

            return top;
        }

        public static int getRank(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            var tulo = db_.ExecuteQuery<PERSONID>
                (@"SELECT userid FROM PERSONID
                WHERE gpd_index < {0}", found.First().gpd_index);

            return tulo.Count() + 1;
        }


        //return a persons first name from the database
        public static string getFirstName(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            return found.First().firstname.ToString();
        }

       
        //return a persons last name from the database
        public static string getLastName(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            return found.First().lastname.ToString();
        }

        //return a persons nationality from the database
        public static string getNationality(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            return found.First().nationality.ToString();
        }

        //return a persons sex from the database
        public static string getSex(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            return found.First().sex.ToString();
        }


        public static setVehicle_ getVehicleInfo(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            setVehicle_ temp = new setVehicle_();

            PERSONID tmp = found.First();

            temp.manufacturer = tmp.vehicle_manufacturer.ToString();
            temp.consumption = (float)tmp.vehicle_consumption;
            temp.fueltype = tmp.vehicle_fueltype.ToString();
            temp.model = tmp.vehicle_model.ToString();
            temp.year = (int)tmp.vehicle_year;

            return temp;
        }



        public static long getPoints(string email)
        {
            var found = db_.ExecuteQuery<PERSONID>(@"SELECT * FROM PERSONID WHERE email = {0}", email);

            return found.First().gpd_index;
        }

    }
}
